﻿-- ClassColor Bars v0.4
-- Written by Killakhan
-- Changes unitframe healthbars to class colored bars
-- http://www.wowinterface.com/downloads/author-259436.html

local ClassColorBars = {}
local addon = ClassColorBars
local addonName = "KCCB"
local version = "0.4"

KCCB_DB = {
	showSelf = false,
	clearBars = false,
}
addon.f = CreateFrame("Frame")
addon.f:SetScript("OnEvent", function(self, event, ...) 
      if addon[event] then 
         return addon[event](addon, event, ...) 
      end 
end)

function addon.ShowSelf()
	if KCCB_DB.showSelf then
		local color = RAID_CLASS_COLORS[select(2, UnitClass("player"))]
		PlayerFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b)		
	else
		PlayerFrameHealthBar:SetStatusBarColor(0, 1, 0) 
	end
end

PlayerFrameHealthBar.lockColor = true
TargetFrameHealthBar.lockColor = true
TargetFrameToTHealthBar.lockColor = true
FocusFrameHealthBar.lockColor = true
FocusFrameToTHealthBar.lockColor = true
for i = 1, 4 do
	_G["PartyMemberFrame"..i.."HealthBar"].lockColor = nil
end	
	
-- register some events
addon.f:RegisterEvent("ADDON_LOADED")
addon.f:RegisterEvent("PLAYER_ENTERING_WORLD")
addon.f:RegisterEvent("PARTY_MEMBERS_CHANGED")
addon.f:RegisterEvent("PLAYER_TARGET_CHANGED")
addon.f:RegisterEvent("PLAYER_REGEN_ENABLED")
addon.f:RegisterEvent("PLAYER_REGEN_DISABLED")

hooksecurefunc("HealthBar_OnValueChanged", function(self)
	if not (self.unit == "party1" or self.unit == "party2" or self.unit == "party3" or self.unit == "party4") then
		return 
	end
	if UnitExists(self.unit) and UnitClass(self.unit) and UnitIsConnected(self.unit) then
		-- print("set color on: " .. self.unit)
		local color = RAID_CLASS_COLORS[select(2, UnitClass(self.unit))]
		self:SetStatusBarColor(color.r, color.g, color.b)		
	end
end)

function addon:ADDON_LOADED(event, addon)
	if addon == "ClassColorBars" then
		-- initialize config
		self.checkbox1:SetChecked(KCCB_DB.showSelf)
		if KCCB_DB.showSelf then
	        local color = RAID_CLASS_COLORS[select(2, UnitClass("player"))]
			PlayerFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b)	
		else
			PlayerFrameHealthBar:SetStatusBarColor(0, 1, 0) 
		end
		self.checkbox2:SetChecked(KCCB_DB.clearBars)
		if KCCB_DB.clearBars then
	        self.SetBarTexture("clear")
		else
			self.SetBarTexture("normal")
		end
	end
end

function addon:PLAYER_REGEN_ENABLED(event)
	-- print("lock color true")
	for i = 1, 4 do
		_G["PartyMemberFrame"..i.."HealthBar"].lockColor = true
	end	
end
function addon:PLAYER_REGEN_DISABLED(event)
	-- print("lock color false")
	for i = 1, 4 do
		_G["PartyMemberFrame"..i.."HealthBar"].lockColor = nil
	end	
end

function addon:PLAYER_TARGET_CHANGED(event, ...)
	if not UnitExists("target") then return end
	if UnitIsPlayer("target") then
		local color = RAID_CLASS_COLORS[select(2, UnitClass("target"))]
		TargetFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b)
	else
		TargetFrameHealthBar:SetStatusBarColor(0, 1, 0)
	end
	if UnitExists("targettarget") and UnitIsPlayer("targettarget") then
		local color = RAID_CLASS_COLORS[select(2, UnitClass("targettarget"))]
		TargetFrameToTHealthBar:SetStatusBarColor(color.r, color.g, color.b)
	else
		TargetFrameToTHealthBar:SetStatusBarColor(0, 1, 0)
	end
end

addon.f:RegisterEvent("PLAYER_FOCUS_CHANGED")
function addon:PLAYER_FOCUS_CHANGED(event, ...)
	if not UnitExists("focus") then return end
	if UnitIsPlayer("focus") then
		local color = RAID_CLASS_COLORS[select(2, UnitClass("focus"))]
		FocusFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b)
	else
		FocusFrameHealthBar:SetStatusBarColor(0, 1, 0)
	end
	if UnitExists("focustarget") and UnitIsPlayer("focustarget") then
		local color = RAID_CLASS_COLORS[select(2, UnitClass("focustarget"))]
		FocusFrameToTHealthBar:SetStatusBarColor(color.r, color.g, color.b)
	else
		FocusFrameToTHealthBar:SetStatusBarColor(0, 1, 0)
	end
end

function addon:PARTY_MEMBERS_CHANGED(event, ...)
	-- if UnitInRaid("player") then return end
	-- print("event")
	for i = 1, 4 do
		local unit = "party"..i
		if UnitExists(unit) and UnitClass(unit) then
			local color = RAID_CLASS_COLORS[select(2, UnitClass(unit))]
			_G["PartyMemberFrame"..i.."HealthBar"]:SetStatusBarColor(color.r, color.g, color.b)
		end
	end
end
addon.PLAYER_ENTERING_WORLD = addon.PARTY_MEMBERS_CHANGED

addon.f:RegisterEvent("UNIT_TARGET")
function addon:UNIT_TARGET(event, unit)
	if not unit == "focus" or not unit == "target" then return end
	if unit == "target" then
		local color = RAID_CLASS_COLORS[select(2, UnitClass("targettarget"))]
		if color and UnitIsPlayer("targettarget") then
			TargetFrameToTHealthBar:SetStatusBarColor(color.r, color.g, color.b)
		else
			TargetFrameToTHealthBar:SetStatusBarColor(0, 1, 0)
		end
	elseif unit == "focus" then
		local color = RAID_CLASS_COLORS[select(2, UnitClass("focustarget"))]
		if color and UnitIsPlayer("focustarget") then
			FocusFrameToTHealthBar:SetStatusBarColor(color.r, color.g, color.b)
		else
			FocusFrameToTHealthBar:SetStatusBarColor(0, 1, 0)
		end
	end
end


addon.panel = CreateFrame("Frame", addonName.."panel", UIParent)
addon.panel.name = "ClassColor Bars"
InterfaceOptions_AddCategory(addon.panel)

addon.title = addon.panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
addon.title:SetPoint("TOPLEFT", 16, -16)
addon.title:SetText("ClassColor Bars Config")

addon.subtitle = addon.panel:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
addon.subtitle:SetPoint("TOPLEFT", addon.title, "BOTTOMLEFT", 0, -8)
addon.subtitle:SetNonSpaceWrap(true)
addon.subtitle:SetText("v"..version)
addon.subtitle:SetTextColor(1, 1, 1, 0.5)

addon.checkbox1 = CreateFrame("CheckButton", addonName.."btn1", addon.panel, "ChatConfigCheckButtonTemplate")
addon.checkbox1:SetPoint("TOPLEFT", addon.subtitle, "BOTTOMLEFT", -5, -20)
_G[addon.checkbox1:GetName().."Text"]:SetText("Show colorbar on self")
_G[addon.checkbox1:GetName().."Text"]:SetPoint("LEFT", addon.checkbox1, "RIGHT", 3, 2)
addon.checkbox1.tooltip = "Click to enable/disable classcolor bar on yourself"
addon.checkbox1:SetChecked(KCCB_DB.showSelf)
addon.checkbox1:SetScript("OnClick", function(self, button, down)
	if addon.checkbox1:GetChecked() then
        local color = RAID_CLASS_COLORS[select(2, UnitClass("player"))]
		PlayerFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b)	
		KCCB_DB.showSelf = true
    else
		PlayerFrameHealthBar:SetStatusBarColor(0, 1, 0) 
		KCCB_DB.showSelf = false
    end
end)

addon.checkbox2 = CreateFrame("CheckButton", addonName.."btn2", addon.panel, "ChatConfigCheckButtonTemplate")
addon.checkbox2:SetPoint("TOPLEFT", addon.checkbox1, "BOTTOMLEFT", 0, -4)
_G[addon.checkbox2:GetName().."Text"]:SetText("Use clear bartexture")
_G[addon.checkbox2:GetName().."Text"]:SetPoint("LEFT", addon.checkbox2, "RIGHT", 3, 2)
addon.checkbox2.tooltip = "Click to enable/disable clear statusbars"
addon.checkbox2:SetChecked(KCCB_DB.clearBars)
addon.checkbox2:SetScript("OnClick", function(self, button, down)
	if addon.checkbox2:GetChecked() then
        addon.SetBarTexture("clear")
		KCCB_DB.clearBars = true
    else
		addon.SetBarTexture("normal")
		KCCB_DB.clearBars = false
    end
end)

function addon.SetBarTexture(flag)
	local texture 
	if flag == "clear" then
		texture = "Interface\\Addons\\ClassColorBars\\gfx\\minim2"
	else
		texture = "Interface\\TargetingFrame\\UI-StatusBar"
	end
		
	PlayerFrameHealthBar:SetStatusBarTexture(texture)
	PlayerFrameManaBar:SetStatusBarTexture(texture)

	FocusFrameHealthBar:SetStatusBarTexture(texture)
	FocusFrameManaBar:SetStatusBarTexture(texture)

	TargetFrameHealthBar:SetStatusBarTexture(texture)
	TargetFrameManaBar:SetStatusBarTexture(texture)

	TargetFrameToTHealthBar:SetStatusBarTexture(texture)
	TargetFrameToTManaBar:SetStatusBarTexture(texture)

	FocusFrameToTHealthBar:SetStatusBarTexture(texture)
	FocusFrameToTManaBar:SetStatusBarTexture(texture)

	PetFrameHealthBar:SetStatusBarTexture(texture)
	PetFrameManaBar:SetStatusBarTexture(texture)

	for i = 1, 4 do
		_G["PartyMemberFrame"..i.."HealthBar"]:SetStatusBarTexture(texture)
		_G["PartyMemberFrame"..i.."ManaBar"]:SetStatusBarTexture(texture)
		_G["PartyMemberFrame"..i.."PetFrameHealthBar"]:SetStatusBarTexture(texture)
	end

end


