if not _G[...] then return end
local addon = _G[...]
local title = addon.title
setfenv(1, addon)

local info, entry, isPanel, open = { }

--[[-----------------------------------------------------------------------------
Core
-------------------------------------------------------------------------------]]
local function DeletePanel(id)
	local cacheSettings, pluginSettings, children, name, plugin  = settings.panels[id], settings.plugins
	settings.panels[id] = nil
	if settings.defaultPanel == id then
		settings.defaultPanel = next(settings.panels)
	end
	for section in pairs(sectionTypes) do
		children = cacheSettings[section]
		for index = 1, #children do
			name = children[index]
			if pluginSettings[name] then
				pluginSettings[name].enable = false
				SetInheritance(name, nil)
			end
		end
	end
	if panels[id] then
		panels[id]:Recycle()
	end
	PanelList:Delete(id)
	update_panel_list()
end

local function DeletePlugin(name)
	local id, section, index = GetPluginLocation(name)
	if id then
		tremove(settings.panels[id][section], index)
	end
	settings.plugins[name] = nil
	PluginList:Delete(name)
end

local function RenamePanel(id, value)
	if value == "" or value == L["<unnamed>"] then
		value = nil
	end
	if settings.panels[id].alias ~= value then
		settings.panels[id].alias = value
		PanelList:Add(id)
		update_panel_list()
	end
end

local function RenamePlugin(name, value)
	if value == "" or value == name then
		value = nil
	end
	if pluginAlias[name] ~= value then
		pluginAlias[name] = value
		if plugins[name] then
			plugins[name]:Refresh()
		end
		PluginList:Add(name)
	end
end

--[[-----------------------------------------------------------------------------
Edit box
-------------------------------------------------------------------------------]]
local editBox = CreateFrame('EditBox', nil, SideBar)
editBox:SetFrameLevel(editBox:GetFrameLevel() + 1)
editBox:SetCountInvisibleLetters(false)
editBox:Hide()

editBox:SetScript('OnEditFocusLost', function(self)
	if self:IsShown() then
		if self.ignore then
			self.ignore = nil
		else
			(isPanel and RenamePanel or RenamePlugin)(self.entry.data.key, self:GetText():trim())
		end
		self:Hide()
	end
end)

editBox:SetScript('OnEnterPressed', editBox.ClearFocus)

editBox:SetScript('OnEscapePressed', function(self)
	self.ignore = true
	self:ClearFocus()
end)

editBox:SetScript('OnHide', function(self)
	self.entry.text:Show()
end)

editBox:SetScript('OnShow', function(self)
	self.entry = entry
	entry.text:Hide()
	self:SetAllPoints(entry.text)
	self:SetFontObject(entry:GetNormalFontObject())
	self:SetText(entry.data.label)
	self:HighlightText()
	self:SetCursorPosition(self:GetNumLetters())
	self:SetFocus()
end)

--[[-----------------------------------------------------------------------------
Drop down menu
-------------------------------------------------------------------------------]]
local function delete_entry()
	editBox:ClearFocus()
	return (isPanel and DeletePanel or DeletePlugin)(entry.data.key)
end

local function rename_entry()
	editBox:ClearFocus()
	editBox:Show()
end

SideBarMenu = {
	displayMode = 'MENU',

	initialize = function(self, level)
		wipe(info)
		info.notCheckable = true

		info.isTitle = true
		info.text = RemoveColorCodes(entry.data.label)
		UIDropDownMenu_AddButton(info, level)
		info.isTitle = nil

		info.text = L["Delete"]
		info.func = delete_entry
		if isPanel then
			info.disabled = #PanelList <= 1
		else
			info.disabled = dataObj[entry.data.key]
		end
		UIDropDownMenu_AddButton(info, level)

		info.text = L["Rename"]
		info.func = rename_entry
		info.disabled = nil
		UIDropDownMenu_AddButton(info, level)
	end,

	onHide = function()
		open = nil
	end,

	Close = function(self)
		if open then
			ToggleDropDownMenu(1, nil, self, 'cursor')
		end
	end,

	Open = function(self, button, currentList)
		entry, isPanel = button, currentList == PanelList
		self:Close()
		ToggleDropDownMenu(1, nil, self, 'cursor')
		open = true
	end,

	SetHeight = DoNothing
}
